# Generated by roxygen2: do not edit by hand

export(analyze_temporal_patterns)
export(analyze_trends_by_spatial_unit)
export(build_temporal_gam)
export(build_temporal_glm)
export(build_temporal_hv)
export(build_temporal_rf)
export(generate_absences)
export(generate_spatiotemporal_predictions)
export(plot_model_assessment)
export(raster_align)
export(scale_rasters)
export(spatiotemporal_partition)
export(spatiotemporal_rarefaction)
export(summarize_raster_outputs)
export(temporally_explicit_extraction)
importFrom(deldir,deldir)
importFrom(deldir,tile.list)
importFrom(exactextractr,exact_extract)
importFrom(grDevices,adjustcolor)
importFrom(grDevices,colorRampPalette)
importFrom(grDevices,dev.cur)
importFrom(grDevices,dev.off)
importFrom(grDevices,hcl.colors)
importFrom(grDevices,hcl.pals)
importFrom(grDevices,heat.colors)
importFrom(grDevices,png)
importFrom(grDevices,rainbow)
importFrom(grDevices,recordPlot)
importFrom(grDevices,terrain.colors)
importFrom(graphics,abline)
importFrom(graphics,axis)
importFrom(graphics,barplot)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,mtext)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(graphics,rect)
importFrom(graphics,rug)
importFrom(sf,sf_use_s2)
importFrom(sf,st_as_sf)
importFrom(sf,st_as_sfc)
importFrom(sf,st_bbox)
importFrom(sf,st_buffer)
importFrom(sf,st_cast)
importFrom(sf,st_collection_extract)
importFrom(sf,st_contains)
importFrom(sf,st_convex_hull)
importFrom(sf,st_coordinates)
importFrom(sf,st_crs)
importFrom(sf,st_difference)
importFrom(sf,st_distance)
importFrom(sf,st_drop_geometry)
importFrom(sf,st_geometry)
importFrom(sf,st_geometry_type)
importFrom(sf,st_intersection)
importFrom(sf,st_intersects)
importFrom(sf,st_is_empty)
importFrom(sf,st_join)
importFrom(sf,st_point)
importFrom(sf,st_point_on_surface)
importFrom(sf,st_polygon)
importFrom(sf,st_read)
importFrom(sf,st_sample)
importFrom(sf,st_sf)
importFrom(sf,st_sfc)
importFrom(sf,st_transform)
importFrom(sf,st_union)
importFrom(sf,st_voronoi)
importFrom(stats,as.formula)
importFrom(stats,binomial)
importFrom(stats,chisq.test)
importFrom(stats,coef)
importFrom(stats,complete.cases)
importFrom(stats,dbinom)
importFrom(stats,dist)
importFrom(stats,glm)
importFrom(stats,kmeans)
importFrom(stats,logLik)
importFrom(stats,median)
importFrom(stats,pchisq)
importFrom(stats,pnorm)
importFrom(stats,predict)
importFrom(stats,quantile)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,var)
importFrom(terra,app)
importFrom(terra,classify)
importFrom(terra,crop)
importFrom(terra,crs)
importFrom(terra,ext)
importFrom(terra,extract)
importFrom(terra,focal)
importFrom(terra,freq)
importFrom(terra,global)
importFrom(terra,is.related)
importFrom(terra,mask)
importFrom(terra,mosaic)
importFrom(terra,ncell)
importFrom(terra,nlyr)
importFrom(terra,plot)
importFrom(terra,project)
importFrom(terra,rast)
importFrom(terra,res)
importFrom(terra,resample)
importFrom(terra,values)
importFrom(terra,vect)
importFrom(terra,writeRaster)
importFrom(terra,xyFromCell)
importFrom(tools,file_ext)
importFrom(tools,file_path_sans_ext)
importFrom(tools,toTitleCase)
importFrom(utils,capture.output)
importFrom(utils,combn)
importFrom(utils,head)
importFrom(utils,modifyList)
importFrom(utils,read.csv)
importFrom(utils,setTxtProgressBar)
importFrom(utils,txtProgressBar)
importFrom(utils,write.csv)
