% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_documentation.R
\docType{data}
\name{tmr_absences_user}
\alias{tmr_absences_user}
\title{Pre-built user_data pseudoabsences (seasonal workflow)}
\format{
A list as returned by \code{\link{generate_absences}}, containing
  \code{$pseudoabsences} (an sf object with attached predictor columns),
  \code{$plots}, and \code{$summary}.
}
\usage{
tmr_absences_user
}
\description{
Output of \code{\link{generate_absences}} run with
\code{method = "user_data"} against \code{\link{tmr_partition}}, using
\code{inst/extdata/points/synthetic_user_presences.csv} as the supplied
absence locations. The CSV contains occurrence-formatted points for a
'related speecies' to that data of interest to be used as a proxy for survey
effort for the species of interest, serving as an alternative method for
generating pseudoabsences.
}
\keyword{datasets}
