% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_documentation.R
\docType{data}
\name{tmr_predictions_annual}
\alias{tmr_predictions_annual}
\title{Pre-built spatiotemporal predictions (annual workflow)}
\format{
A list as returned by
  \code{\link{generate_spatiotemporal_predictions}}.
}
\usage{
tmr_predictions_annual
}
\description{
Annual variant of \code{tmr_predictions}. Projects \code{tmr_glm_annual}
across 15 years on a single (year-only) time axis. Loaded with
\code{data(tmr_predictions_annual)}.
}
\keyword{datasets}
